/*
   This file is part of TALER
   Copyright (C) 2022, 2023 Taler Systems SA

   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.

   TALER is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
/**
 * @file src/backenddb/iterate_wirewatch_accounts.c
 * @brief Implementation of the iterate_wirewatch_accounts function for Postgres
 * @author Christian Grothoff
 */
#include "platform.h"
#include <taler/taler_pq_lib.h>
#include "merchant-database/iterate_wirewatch_accounts.h"
#include "helper.h"


/**
 * Closure for #handle_results().
 */
struct Context
{
  /**
   * Function to call with results.
   */
  TALER_MERCHANTDB_WirewatchWorkCallback cb;

  /**
   * Closure for @e cb.
   */
  void *cb_cls;

  /**
   * Set to true if the parsing failed.
   */
  bool failure;
};


/**
 * Function to be called with the results of a SELECT statement
 * that has returned @a num_results results about accounts.
 *
 * @param cls of type `struct Context *`
 * @param result the postgres result
 * @param num_results the number of results in @a result
 */
static void
handle_results (void *cls,
                PGresult *result,
                unsigned int num_results)
{
  struct Context *ctx = cls;

  for (unsigned int i = 0; i < num_results; i++)
  {
    char *instance;
    struct TALER_FullPayto payto_uri;
    char *facade_url;
    json_t *credential = NULL;
    uint64_t last_serial;
    struct GNUNET_PQ_ResultSpec rs[] = {
      GNUNET_PQ_result_spec_string ("out_merchant_id",
                                    &instance),
      GNUNET_PQ_result_spec_string ("out_payto_uri",
                                    &payto_uri.full_payto),
      GNUNET_PQ_result_spec_string ("out_credit_facade_url",
                                    &facade_url),
      GNUNET_PQ_result_spec_allow_null (
        TALER_PQ_result_spec_json ("out_credit_facade_credentials",
                                   &credential),
        NULL),
      GNUNET_PQ_result_spec_uint64 ("out_last_bank_serial",
                                    &last_serial),
      GNUNET_PQ_result_spec_end
    };

    if (GNUNET_OK !=
        GNUNET_PQ_extract_result (result,
                                  rs,
                                  i))
    {
      GNUNET_break (0);
      ctx->failure = true;
      return;
    }
    ctx->cb (ctx->cb_cls,
             instance,
             payto_uri,
             facade_url,
             credential,
             last_serial);
    GNUNET_PQ_cleanup_result (rs);
  }
}


enum GNUNET_DB_QueryStatus
TALER_MERCHANTDB_iterate_wirewatch_accounts (
  struct TALER_MERCHANTDB_PostgresContext *pg,
  TALER_MERCHANTDB_WirewatchWorkCallback cb,
  void *cb_cls)
{
  struct Context ctx = {
    .cb = cb,
    .cb_cls = cb_cls
  };
  struct GNUNET_PQ_QueryParam params[] = {
    GNUNET_PQ_query_param_end
  };
  enum GNUNET_DB_QueryStatus qs;

  PREPARE (pg,
           "iterate_wirewatch_accounts",
           "SELECT"
           "  out_last_bank_serial"
           " ,out_merchant_id"
           " ,out_payto_uri"
           " ,out_credit_facade_url"
           " ,out_credit_facade_credentials::TEXT"
           " FROM merchant.select_wirewatch_accounts()");
  qs = GNUNET_PQ_eval_prepared_multi_select (pg->conn,
                                             "iterate_wirewatch_accounts",
                                             params,
                                             &handle_results,
                                             &ctx);
  if (ctx.failure)
    return GNUNET_DB_STATUS_HARD_ERROR;
  return qs;
}
